#include <vector>

#include<vcg/simplex/vertexplus/base.h>
#include<vcg/simplex/faceplus/base.h>
#include<vcg/simplex/face/topology.h>
#include<vcg/complex/trimesh/base.h>
#include<wrap/io_trimesh/import_ply.h>

// topology computation
#include<vcg/complex/trimesh/update/topology.h>
#include<vcg/complex/trimesh/update/bounding.h>
#include<vcg/complex/trimesh/update/flag.h>
#include<vcg/complex/trimesh/clean.h>

// half edge iterators
#include<vcg/simplex/face/pos.h>


using namespace vcg;

class MyEdge;    // dummy prototype never used
class MyFace;
class MyVertex;

class MyVertex  : public VertexSimp2< MyVertex, MyEdge, MyFace, vert::Coord3f, vert::BitFlags  >{};
//class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef,face::FFAdj, face::Mark, face::BitFlags > {};
class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef, face::Mark, face::BitFlags > {};

class MyMesh : public tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace > >{};


int main(int argc,char **argv ){

	MyMesh m;

	if(tri::io::ImporterPLY<MyMesh>::Open(m,argv[1])!=0)
		{
      printf("Error reading file  %s\n",argv[1]);
			exit(0);
		}


  printf("Mesh has %i vert and %i faces\n", m.vn,m.fn);
  printf("Mesh is %f x %f x %f \n", m.bbox.max[0]-m.bbox.min[0],
                                    m.bbox.max[1]-m.bbox.min[1],
                                    m.bbox.max[2]-m.bbox.min[2]);
  tri::UpdateBounding<MyMesh>::Box(m);
  printf("Mesh is %f x %f x %f \n", m.bbox.DimX(),m.bbox.DimY(),m.bbox.DimZ());

  tri::UpdateFlags<MyMesh>::FaceBorderFromNone(m);

  int borderEdgeCnt=0;
  MyMesh::FaceIterator fi;
  for(fi=m.face.begin();fi!=m.face.end();++fi)
    if(!(*fi).IsD()) 
    {
      for(int i=0;i<3;++i)
        if((*fi).IsB(i)) ++borderEdgeCnt;
    //  if(IsBorder(*fi,i)) ++borderEdgeCnt;
    }

  printf("Border Edge Count %i\n",borderEdgeCnt);
  int internalEdgeCnt=(m.fn*3 - borderEdgeCnt)/2;
  printf("Internal Edge Count %i\n",internalEdgeCnt);


  printf("%i - %i + %i = %i\n",m.vn, internalEdgeCnt+borderEdgeCnt,m.fn, 
                               m.vn-(internalEdgeCnt+borderEdgeCnt)+m.fn);

  int delCnt=tri::Clean<MyMesh>::RemoveUnreferencedVertex( m);

  printf("Deleted %i unreferenced vertices\n",delCnt);
  printf("%i - %i + %i = %i\n",m.vn, internalEdgeCnt+borderEdgeCnt,m.fn, 
                               m.vn-(internalEdgeCnt+borderEdgeCnt)+m.fn);

  
  return 0;
}

